@@ -15,7 +15,7 @@ from rss_tube.database.settings import Settings
 from rss_tube.gui.EntryWidgets.BaseEntry import BaseEntry
 from rss_tube.updater import Updater
 from rss_tube.utils import get_abs_path, set_icons, set_style
-from rss_tube.tasks import Tasks, AddFeedTask, ExportFeedsTask, ImportFeedsTask
+from rss_tube.tasks import Tasks, AddFeedTask, ExportFeedsTask, ImportFeedsTask, ImportFiltersTask, ExportFiltersTask
 from .AboutDialog import AboutDialog
 from .RenameDialog import RenameCategoryDialog, RenameChannelDialog
 from .ShortcutDialog import ShortcutsDialog
@@ -159,6 +159,11 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow, QtCore.QCoreApplication):
 
     def settings_callback(self):
         settings_dialog = SettingsDialog(self)
+        settings_dialog.quit_requested.connect(self.quit)
+        settings_dialog.import_channels.connect(self.import_channels_callback)
+        settings_dialog.export_channels.connect(self.export_channels_callback)
+        settings_dialog.import_filters.connect(self.import_filters_callback)
+        settings_dialog.export_filters.connect(self.export_filters_callback)
         accepted = settings_dialog.exec()
 
         self.set_shortcuts()
@@ -268,7 +273,7 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow, QtCore.QCoreApplication):
                 self.tree_feeds.add_category(category)
 
     def import_channels_callback(self):
-        fname = QtWidgets.QFileDialog.getOpenFileName(self, "Import", settings.value("MainWindow/export_location", type=str), "*")[0]
+        fname = QtWidgets.QFileDialog.getOpenFileName(self, "Import Channels", settings.value("Channels/export_path", type=str), "*")[0]
         if fname and os.path.exists(fname):
             self.import_feeds_task = ImportFeedsTask(fname)
             self.import_feeds_task.imported.connect(self.channel_imported_callback)
@@ -279,11 +284,25 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow, QtCore.QCoreApplication):
         self.tree_feeds.add_feed(feed_id)
 
     def export_channels_callback(self):
-        fname = QtWidgets.QFileDialog.getSaveFileName(self, "Save", settings.value("MainWindow/export_location", type=str), "*")[0]
+        fname = QtWidgets.QFileDialog.getSaveFileName(self, "Export Channels", settings.value("Channels/export_path", type=str), "*")[0]
         if fname and os.path.exists(os.path.dirname(fname)):
             self.export_feeds_task = ExportFeedsTask(fname)
             self.export_feeds_task.start()
-            settings.setValue("MainWindow/export_location", fname)
+            settings.setValue("Channels/export_path", fname)
+    
+    def import_filters_callback(self):
+        fname = QtWidgets.QFileDialog.getOpenFileName(self, "Import Filters", settings.value("Filters/export_path", type=str), "*")[0]
+        if fname and os.path.exists(fname):
+            self.import_filters_task = ImportFiltersTask(fname)
+            self.import_filters_task.imported.connect(self.channel_imported_callback)
+            self.import_filters_task.start()
+
+    def export_filters_callback(self):
+        fname = QtWidgets.QFileDialog.getSaveFileName(self, "Export Filters", settings.value("Filters/export_path", type=str), "*")[0]
+        if fname and os.path.exists(os.path.dirname(fname)):
+            self.export_filters_task = ExportFiltersTask(fname)
+            self.export_filters_task.start()
+            settings.setValue("Channels/export_path", fname)
 
     def update_feeds_callback(self):
         settings.set_last_refresh()
@@ -320,12 +339,6 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow, QtCore.QCoreApplication):
             return
 
         if new_entries_unviewed := self.feeds.get_new_entries_unviewed(last_refresh):
-            # Display a popup notification
-            if settings.value("tray/notifications/enabled", type=bool):
-                titles = ", ".join(x["title"] for x in new_entries_unviewed)
-                logger.debug(f"update_feeds_finished: {len(new_entries_unviewed)} new entries to notify." + titles)
-                self.display_notification(f"{len(new_entries_unviewed)} new entries.")
-
             # Change to tray icon to show that there are new entries
             if settings.value("tray/show", type=bool) and (self.windowState() & QtCore.Qt.WindowState.WindowMinimized):
                 self.tray.setIcon(QtGui.QIcon(get_abs_path(f"rss_tube/gui/themes/{settings.value('theme', type=str)}/tray_new.png")))
@@ -398,15 +411,16 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow, QtCore.QCoreApplication):
 
     def display_notification(self,
                              message: str,
-                             icon: QtWidgets.QSystemTrayIcon.MessageIcon = QtWidgets.QSystemTrayIcon.MessageIcon.Information):
-        if not settings.value("tray/notifications/enabled", type=bool) or not settings.value("tray/show", type=bool):
+                             icon: QtWidgets.QSystemTrayIcon.MessageIcon = QtWidgets.QSystemTrayIcon.MessageIcon.Information,
+                             duration_ms: int = 2000):
+        if not settings.value("tray/show", type=bool):
             return
 
         self.tray.showMessage(
             self.windowTitle(),
             message,
             icon,
-            settings.value("tray/notifications/duration_ms", type=int)
+            duration_ms
         )
 
     def bring_to_front(self):
@@ -536,7 +550,10 @@ def start_gui():
     logdir = QtCore.QStandardPaths.standardLocations(QtCore.QStandardPaths.StandardLocation.AppDataLocation)[0]
     if not os.path.exists(logdir):
         os.mkdir(logdir)
-    logging.basicConfig(filename=os.path.join(logdir, __title__.replace(" ", "-") + ".log"), format='%(levelname)s > %(name)s > %(asctime)s > %(message)s')
+    logging.basicConfig(
+        filename=os.path.join(logdir, __title__.replace(" ", "-") + ".log"),
+        format="%(levelname)s > %(name)s > %(asctime)s > %(filename)20s:%(lineno)3s - %(funcName)20s() > %(message)s",
+    )
 
     window = MainWindow(app)
 
