@@ -1,11 +1,7 @@
 import subprocess
 
-from rss_tube.database.settings import Settings
-
 from .base import BasePlayer
 
-settings = Settings()
-
 
 class VlcPlayerInstance(BasePlayer):
     def _run(self):        
